@@ -55,7 +55,7 @@ function PMA_ifSetOr(&$var, $default = null, $type = 'similar')
  * $type can be:
  * - false       : no type checking
  * - 'scalar'    : whether type of $var is integer, float, string or boolean
- * - 'numeric'   : whether type of $var is any number repesentation
+ * - 'numeric'   : whether type of $var is any number representation
  * - 'length'    : whether type of $var is scalar with a string length > 0
  * - 'similar'   : whether type of $var is similar to type of $compare
  * - 'equal'     : whether type of $var is identical to type of $compare
@@ -201,7 +201,7 @@ function PMA_securePath($path)
  * @param string|array $message_args   arguments applied to $error_message
  * @param boolean      $delete_session whether to delete session cookie
  *
- * @return exit
+ * @return void
  */
 function PMA_fatalError(
     $error_message, $message_args = null, $delete_session = true
@@ -220,12 +220,9 @@ function PMA_fatalError(
     } else {
         $error_message = strtr($error_message, array('<br />' => '[br]'));
 
-        /* Define fake gettext for fatal errors */
+        /* Load gettext for fatal errors */
         if (!function_exists('__')) {
-            function __($text)
-            {
-                return $text;
-            }
+            include_once './libraries/php-gettext/gettext.inc';
         }
 
         // these variables are used in the included file libraries/error.inc.php
@@ -303,15 +300,16 @@ function PMA_warnMissingExtension($extension, $fatal = false, $extra = '')
     }
     if ($fatal) {
         PMA_fatalError($message);
-    } else {
-        $GLOBALS['error_handler']->addError(
-            $message, 
-            E_USER_WARNING,
-            '',
-            '',
-            false
-        );
+        return;
     }
+
+    $GLOBALS['error_handler']->addError(
+        $message,
+        E_USER_WARNING,
+        '',
+        '',
+        false
+    );
 }
 
 /**
@@ -323,13 +321,13 @@ function PMA_warnMissingExtension($extension, $fatal = false, $extra = '')
  */
 function PMA_getTableCount($db)
 {
-    $tables = PMA_DBI_try_query(
+    $tables = $GLOBALS['dbi']->tryQuery(
         'SHOW TABLES FROM ' . PMA_Util::backquote($db) . ';',
-        null, PMA_DBI_QUERY_STORE
+        null, PMA_DatabaseInterface::QUERY_STORE
     );
     if ($tables) {
-        $num_tables = PMA_DBI_num_rows($tables);
-        PMA_DBI_free_result($tables);
+        $num_tables = $GLOBALS['dbi']->numRows($tables);
+        $GLOBALS['dbi']->freeResult($tables);
     } else {
         $num_tables = 0;
     }
@@ -343,7 +341,7 @@ function PMA_getTableCount($db)
  * (renamed with PMA prefix to avoid double definition when embedded
  * in Moodle)
  *
- * @param string $size size
+ * @param string|int $size size (Default = 0)
  *
  * @return integer $size
  */
@@ -353,6 +351,7 @@ function PMA_getRealSize($size = 0)
         return 0;
     }
 
+    $scan = array();
     $scan['gb'] = 1073741824; //1024 * 1024 * 1024;
     $scan['g']  = 1073741824; //1024 * 1024 * 1024;
     $scan['mb'] = 1048576;
@@ -445,10 +444,17 @@ function PMA_arrayMergeRecursive()
 function PMA_arrayWalkRecursive(&$array, $function, $apply_to_keys_also = false)
 {
     static $recursive_counter = 0;
+    $walked_keys = array();
+
     if (++$recursive_counter > 1000) {
         PMA_fatalError(__('possible deep recursion attack'));
     }
     foreach ($array as $key => $value) {
+        if (isset($walked_keys[$key])) {
+            continue;
+        }
+        $walked_keys[$key] = true;
+
         if (is_array($value)) {
             PMA_arrayWalkRecursive($array[$key], $function, $apply_to_keys_also);
         } else {
@@ -460,6 +466,7 @@ function PMA_arrayWalkRecursive(&$array, $function, $apply_to_keys_also = false)
             if ($new_key != $key) {
                 $array[$new_key] = $array[$key];
                 unset($array[$key]);
+                $walked_keys[$new_key] = true;
             }
         }
     }
@@ -469,8 +476,8 @@ function PMA_arrayWalkRecursive(&$array, $function, $apply_to_keys_also = false)
 /**
  * boolean phpMyAdmin.PMA_checkPageValidity(string &$page, array $whitelist)
  *
- * checks given given $page against given $whitelist and returns true if valid
- * it ignores optionaly query paramters in $page (script.php?ignored)
+ * checks given $page against given $whitelist and returns true if valid
+ * it optionally ignores query parameters in $page (script.php?ignored)
  *
  * @param string &$page     page to check
  * @param array  $whitelist whitelist to check page against
@@ -485,14 +492,17 @@ function PMA_checkPageValidity(&$page, $whitelist)
 
     if (in_array($page, $whitelist)) {
         return true;
-    } elseif (in_array(substr($page, 0, strpos($page . '?', '?')), $whitelist)) {
-        return true;
-    } else {
-        $_page = urldecode($page);
-        if (in_array(substr($_page, 0, strpos($_page . '?', '?')), $whitelist)) {
-            return true;
-        }
     }
+
+    if (in_array(substr($page, 0, strpos($page . '?', '?')), $whitelist)) {
+        return true;
+    }
+
+    $_page = urldecode($page);
+    if (in_array(substr($_page, 0, strpos($_page . '?', '?')), $whitelist)) {
+        return true;
+    }
+
     return false;
 }
 
@@ -510,12 +520,19 @@ function PMA_getenv($var_name)
 {
     if (isset($_SERVER[$var_name])) {
         return $_SERVER[$var_name];
-    } elseif (isset($_ENV[$var_name])) {
+    }
+
+    if (isset($_ENV[$var_name])) {
         return $_ENV[$var_name];
-    } elseif (getenv($var_name)) {
+    }
+
+    if (getenv($var_name)) {
         return getenv($var_name);
-    } elseif (function_exists('apache_getenv')
-     && apache_getenv($var_name, true)) {
+    }
+
+    if (function_exists('apache_getenv')
+        && apache_getenv($var_name, true)
+    ) {
         return apache_getenv($var_name, true);
     }
 
@@ -557,36 +574,38 @@ function PMA_sendHeaderLocation($uri, $use_refresh = false)
         echo '//]]>' . "\n";
         echo '</script></body></html>' . "\n";
 
-    } else {
-        if (SID) {
-            if (strpos($uri, '?') === false) {
-                header('Location: ' . $uri . '?' . SID);
-            } else {
-                $separator = PMA_get_arg_separator();
-                header('Location: ' . $uri . $separator . SID);
-            }
+        return;
+    }
+
+    if (SID) {
+        if (strpos($uri, '?') === false) {
+            header('Location: ' . $uri . '?' . SID);
         } else {
-            session_write_close();
-            if (headers_sent()) {
-                if (function_exists('debug_print_backtrace')) {
-                    echo '<pre>';
-                    debug_print_backtrace();
-                    echo '</pre>';
-                }
-                trigger_error(
-                    'PMA_sendHeaderLocation called when headers are already sent!',
-                    E_USER_ERROR
-                );
-            }
-            // bug #1523784: IE6 does not like 'Refresh: 0', it
-            // results in a blank page
-            // but we need it when coming from the cookie login panel)
-            if (PMA_IS_IIS && $use_refresh) {
-                header('Refresh: 0; ' . $uri);
-            } else {
-                header('Location: ' . $uri);
-            }
+            $separator = PMA_URL_getArgSeparator();
+            header('Location: ' . $uri . $separator . SID);
         }
+        return;
+    }
+
+    session_write_close();
+    if (headers_sent()) {
+        if (function_exists('debug_print_backtrace')) {
+            echo '<pre>';
+            debug_print_backtrace();
+            echo '</pre>';
+        }
+        trigger_error(
+            'PMA_sendHeaderLocation called when headers are already sent!',
+            E_USER_ERROR
+        );
+    }
+    // bug #1523784: IE6 does not like 'Refresh: 0', it
+    // results in a blank page
+    // but we need it when coming from the cookie login panel)
+    if (PMA_IS_IIS && $use_refresh) {
+        header('Refresh: 0; ' . $uri);
+    } else {
+        header('Location: ' . $uri);
     }
 }
 
@@ -617,13 +636,14 @@ function PMA_noCacheHeader()
          * Adding Pragma: public fixes this.
          */
         header('Pragma: public');
-    } else {
-        header('Pragma: no-cache'); // HTTP/1.0
-        // test case: exporting a database into a .gz file with Safari
-        // would produce files not having the current time
-        // (added this header for Safari but should not harm other browsers)
-        header('Last-Modified: ' . date(DATE_RFC1123));
+        return;
     }
+
+    header('Pragma: no-cache'); // HTTP/1.0
+    // test case: exporting a database into a .gz file with Safari
+    // would produce files not having the current time
+    // (added this header for Safari but should not harm other browsers)
+    header('Last-Modified: ' . date(DATE_RFC1123));
 }
 
 
@@ -650,12 +670,39 @@ function PMA_downloadHeader($filename, $mimetype, $length = 0, $no_cache = true)
         header('Content-Disposition: attachment; filename="' . $filename . '"');
     }
     header('Content-Type: ' . $mimetype);
+    // inform the server that compression has been done,
+    // to avoid a double compression (for example with Apache + mod_deflate)
+    if (strpos($mimetype, 'gzip') !== false) {
+        header('Content-Encoding: gzip');
+    }
     header('Content-Transfer-Encoding: binary');
     if ($length > 0) {
         header('Content-Length: ' . $length);
     }
 }
 
+/**
+ * Checks whether element given by $path exists in $array.
+ * $path is a string describing position of an element in an associative array,
+ * eg. Servers/1/host refers to $array[Servers][1][host]
+ *
+ * @param string $path  path in the arry
+ * @param array  $array the array
+ *
+ * @return mixed    array element or $default
+ */
+function PMA_arrayKeyExists($path, $array)
+{
+    $keys = explode('/', $path);
+    $value =& $array;
+    foreach ($keys as $key) {
+        if (! isset($value[$key])) {
+            return false;
+        }
+        $value =& $value[$key];
+    }
+    return true;
+}
 
 /**
  * Returns value of an element in $array given by $path.
@@ -757,16 +804,59 @@ function PMA_linkURL($url)
 {
     if (!preg_match('#^https?://#', $url) || defined('PMA_SETUP')) {
         return $url;
-    } else {
-        if (!function_exists('PMA_generate_common_url')) {
-            include_once './libraries/url_generating.lib.php';
-        }
-        $params = array();
-        $params['url'] = $url;
-        return './url.php' . PMA_generate_common_url($params);
     }
+
+    if (!function_exists('PMA_URL_getCommon')) {
+        include_once './libraries/url_generating.lib.php';
+    }
+    $params = array();
+    $params['url'] = $url;
+
+    $url = PMA_URL_getCommon($params);
+    //strip off token and such sensitive information. Just keep url.
+    $arr = parse_url($url);
+    parse_str($arr["query"], $vars);
+    $query = http_build_query(array("url" => $vars["url"]));
+    $url = './url.php?' . $query;
+
+    return $url;
 }
 
+/**
+ * Checks whether domain of URL is whitelisted domain or not.
+ * Use only for URLs of external sites.
+ *
+ * @param string $url URL of external site.
+ *
+ * @return boolean.True:if domain of $url is allowed domain, False:otherwise.
+ */
+function PMA_isAllowedDomain($url)
+{
+    $arr = parse_url($url);
+    $domain = $arr["host"];
+    $domainWhiteList = array(
+        /* Include current domain */
+        $_SERVER['SERVER_NAME'],
+        /* phpMyAdmin domains */
+        'wiki.phpmyadmin.net', 'www.phpMyAdmin.net', 'phpmyadmin.net',
+        'docs.phpmyadmin.net',
+        /* mysql.com domains */
+        'dev.mysql.com','bugs.mysql.com',
+        /* php.net domains */
+        'php.net',
+        /* Github domains*/
+        'github.com','www.github.com',
+        /* Following are doubtful ones. */
+        'www.primebase.com','pbxt.blogspot.com'
+    );
+    if (in_array($domain, $domainWhiteList)) {
+        return true;
+    }
+
+    return false;
+}
+
+
 /**
  * Adds JS code snippets to be displayed by the PMA_Response class.
  * Adds a newline to each snippet.
