@@ -128,11 +128,11 @@ class ImportCsv extends AbstractImportCsv
         $csv_new_line = strtr($csv_new_line, $replacements);
 
         $param_error = false;
-        if (strlen($csv_terminated) != 1) {
+        if (strlen($csv_terminated) < 1) {
             $message = PMA_Message::error(
                 __('Invalid parameter for CSV import: %s')
             );
-            $message->addParam(__('Columns terminated by'), false);
+            $message->addParam(__('Columns terminated with'), false);
             $error = true;
             $param_error = true;
             // The default dialog of MS Excel when generating a CSV produces a
@@ -147,21 +147,21 @@ class ImportCsv extends AbstractImportCsv
             $message = PMA_Message::error(
                 __('Invalid parameter for CSV import: %s')
             );
-            $message->addParam(__('Columns enclosed by'), false);
+            $message->addParam(__('Columns enclosed with'), false);
             $error = true;
             $param_error = true;
         } elseif (strlen($csv_escaped) != 1) {
             $message = PMA_Message::error(
                 __('Invalid parameter for CSV import: %s')
             );
-            $message->addParam(__('Columns escaped by'), false);
+            $message->addParam(__('Columns escaped with'), false);
             $error = true;
             $param_error = true;
         } elseif (strlen($csv_new_line) != 1 && $csv_new_line != 'auto') {
             $message = PMA_Message::error(
                 __('Invalid parameter for CSV import: %s')
             );
-            $message->addParam(__('Lines terminated by'), false);
+            $message->addParam(__('Lines terminated with'), false);
             $error = true;
             $param_error = true;
         }
@@ -186,7 +186,7 @@ class ImportCsv extends AbstractImportCsv
             }
             $sql_template .= ' INTO ' . PMA_Util::backquote($table);
 
-            $tmp_fields = PMA_DBI_get_columns($db, $table);
+            $tmp_fields = $GLOBALS['dbi']->getColumns($db, $table);
 
             if (empty($csv_columns)) {
                 $fields = $tmp_fields;
@@ -233,6 +233,7 @@ class ImportCsv extends AbstractImportCsv
         // Defaults for parser
         $i = 0;
         $len = 0;
+        $lastlen = null;
         $line = 1;
         $lasti = -1;
         $values = array();
@@ -245,12 +246,12 @@ class ImportCsv extends AbstractImportCsv
 
         $col_count = 0;
         $max_cols = 0;
-
+        $csv_terminated_len = strlen($csv_terminated);
         while (! ($finished && $i >= $len) && ! $error && ! $timeout_passed) {
             $data = PMA_importGetNextChunk();
             if ($data === false) {
                 // subtract data we didn't handle yet and stop processing
-                $offset -= strlen($buffer);
+                $GLOBALS['offset'] -= strlen($buffer);
                 break;
             } elseif ($data === true) {
                 // Handle rest of buffer
@@ -258,6 +259,22 @@ class ImportCsv extends AbstractImportCsv
                 // Append new data to buffer
                 $buffer .= $data;
                 unset($data);
+
+                // Force a trailing new line at EOF to prevent parsing problems
+                if ($finished && $buffer) {
+                    $finalch = substr($buffer, -1);
+                    if ($csv_new_line == 'auto'
+                        && $finalch != "\r"
+                        && $finalch != "\n"
+                    ) {
+                        $buffer .= "\n";
+                    } elseif ($csv_new_line != 'auto'
+                        && $finalch != $csv_new_line
+                    ) {
+                        $buffer .= $csv_new_line;
+                    }
+                }
+
                 // Do not parse string when we're not at the end
                 // and don't have new line inside
                 if (($csv_new_line == 'auto'
@@ -274,6 +291,13 @@ class ImportCsv extends AbstractImportCsv
             $len = strlen($buffer);
             // Currently parsed char
             $ch = $buffer[$i];
+            if ($csv_terminated_len > 1 && $ch == $csv_terminated[0]) {
+                $ch = $this->readCsvTerminatedString(
+                    $buffer, $ch, $i, $csv_terminated_len
+                );
+                $i += $csv_terminated_len-1;
+
+            }
             while ($i < $len) {
                 // Deadlock protection
                 if ($lasti == $i && $lastlen == $len) {
@@ -297,6 +321,12 @@ class ImportCsv extends AbstractImportCsv
                         $values[] = '';
                         $i++;
                         $ch = $buffer[$i];
+                        if ($csv_terminated_len > 1 && $ch == $csv_terminated[0]) {
+                            $ch = $this->readCsvTerminatedString(
+                                $buffer, $ch, $i, $csv_terminated_len
+                            );
+                            $i += $csv_terminated_len-1;
+                        }
                         continue;
                     }
 
@@ -309,6 +339,12 @@ class ImportCsv extends AbstractImportCsv
                         $need_end = true;
                         $i++;
                         $ch = $buffer[$i];
+                        if ($csv_terminated_len > 1 && $ch == $csv_terminated[0]) {
+                            $ch = $this->readCsvTerminatedString(
+                                $buffer, $ch, $i, $csv_terminated_len
+                            );
+                            $i += $csv_terminated_len-1;
+                        }
                     } else {
                         $need_end = false;
                     }
@@ -329,6 +365,14 @@ class ImportCsv extends AbstractImportCsv
                             }
                             $i++;
                             $ch = $buffer[$i];
+                            if ($csv_terminated_len > 1
+                                && $ch == $csv_terminated[0]
+                            ) {
+                                $ch = $this->readCsvTerminatedString(
+                                    $buffer, $ch, $i, $csv_terminated_len
+                                );
+                                $i += $csv_terminated_len-1;
+                            }
                             if ($csv_enclosed == $csv_escaped
                                 && ($ch == $csv_terminated
                                 || $ch == $csv_new_line
@@ -347,6 +391,12 @@ class ImportCsv extends AbstractImportCsv
                         }
                         $i++;
                         $ch = $buffer[$i];
+                        if ($csv_terminated_len > 1 && $ch == $csv_terminated[0]) {
+                            $ch = $this->readCsvTerminatedString(
+                                $buffer, $ch, $i, $csv_terminated_len
+                            );
+                            $i += $csv_terminated_len-1;
+                        }
                     }
 
                     // unquoted NULL string
@@ -357,6 +407,12 @@ class ImportCsv extends AbstractImportCsv
                     if ($fail) {
                         $i = $fallbacki;
                         $ch = $buffer[$i];
+                        if ($csv_terminated_len > 1 && $ch == $csv_terminated[0]) {
+                            $ch = $this->readCsvTerminatedString(
+                                $buffer, $ch, $i, $csv_terminated_len
+                            );
+                            $i += $csv_terminated_len-1;
+                        }
                         break;
                     }
                     // Need to strip trailing enclosing char?
@@ -366,10 +422,26 @@ class ImportCsv extends AbstractImportCsv
                         } elseif ($i == $len - 1) {
                             $i = $fallbacki;
                             $ch = $buffer[$i];
+                            if ($csv_terminated_len > 1
+                                && $ch == $csv_terminated[0]
+                            ) {
+                                $ch = $this->readCsvTerminatedString(
+                                    $buffer, $ch, $i, $csv_terminated_len
+                                );
+                                $i += $csv_terminated_len-1;
+                            }
                             break;
                         } else {
                             $i++;
                             $ch = $buffer[$i];
+                            if ($csv_terminated_len > 1
+                                && $ch == $csv_terminated[0]
+                            ) {
+                                $ch = $this->readCsvTerminatedString(
+                                    $buffer, $ch, $i, $csv_terminated_len
+                                );
+                                $i += $csv_terminated_len-1;
+                            }
                         }
                     }
                     // Are we at the end?
@@ -384,10 +456,26 @@ class ImportCsv extends AbstractImportCsv
                         if ($i == $len - 1) {
                             $i = $fallbacki;
                             $ch = $buffer[$i];
+                            if ($csv_terminated_len > 1
+                                && $ch == $csv_terminated[0]
+                            ) {
+                                $ch = $this->readCsvTerminatedString(
+                                    $buffer, $ch, $i, $csv_terminated_len
+                                );
+                                $i += $csv_terminated_len-1;
+                            }
                             break;
                         }
                         $i++;
                         $ch = $buffer[$i];
+                        if ($csv_terminated_len > 1
+                            && $ch == $csv_terminated[0]
+                        ) {
+                            $ch = $this->readCsvTerminatedString(
+                                $buffer, $ch, $i, $csv_terminated_len
+                            );
+                            $i += $csv_terminated_len-1;
+                        }
                     }
                     // If everything went okay, store value
                     $values[] = $value;
@@ -434,7 +522,10 @@ class ImportCsv extends AbstractImportCsv
                                 unset($values[count($values) - 1]);
                             } else {
                                 $message = PMA_Message::error(
-                                    __('Invalid column count in CSV input on line %d.')
+                                    __(
+                                        'Invalid column count in CSV input'
+                                        . ' on line %d.'
+                                    )
                                 );
                                 $message->addParam($line);
                                 $error = true;
@@ -498,13 +589,13 @@ class ImportCsv extends AbstractImportCsv
             ) {
                 // Fill out column names
                 for ($i = 0; $i < $max_cols; ++$i) {
-                    $col_names[] = 'COL '.($i+1);
+                    $col_names[] = 'COL ' . ($i+1);
                 }
             }
 
             if (strlen($db)) {
-                $result = PMA_DBI_fetch_result('SHOW TABLES');
-                $tbl_name = 'TABLE '.(count($result) + 1);
+                $result = $GLOBALS['dbi']->fetchResult('SHOW TABLES');
+                $tbl_name = 'TABLE ' . (count($result) + 1);
             } else {
                 $tbl_name = 'TBL_NAME';
             }
@@ -560,7 +651,31 @@ class ImportCsv extends AbstractImportCsv
         }
     }
 
+    /**
+     * Read the expected column_separated_with String of length
+     * $csv_terminated_len from the $buffer
+     * into variable $ch and return the read string $ch
+     *
+     * @param string $buffer             The original string buffer read from
+     *                                   csv file
+     * @param string $ch                 Partially read "column Separated with"
+     *                                   string, also used to return after
+     *                                   reading length equal $csv_terminated_len
+     * @param int    $i                  Current read counter of buffer string
+     * @param int    $csv_terminated_len The length of "column separated with"
+     *                                   String
+     *
+     * @return string
+     */
 
+    public function readCsvTerminatedString($buffer, $ch, $i, $csv_terminated_len)
+    {
+        for ($j = 0; $j < $csv_terminated_len - 1; $j++) {
+            $i++;
+            $ch .= $buffer[$i];
+        }
+        return $ch;
+    }
     /* ~~~~~~~~~~~~~~~~~~~~ Getters and Setters ~~~~~~~~~~~~~~~~~~~~ */
 
 
@@ -585,4 +700,5 @@ class ImportCsv extends AbstractImportCsv
     {
         $this->_analyze = $analyze;
     }
+
 }
