@@ -70,15 +70,22 @@ if (! defined('PMA_MINIMUM_COMMON') && ! defined('TESTSUITE')) {
     padding: .3em;
 }
 
-#pma_navigation div#recentTableList {
+#pma_navigation div#recentTableList,
+#pma_navigation div#FavoriteTableList {
     text-align: center;
     margin-bottom: 0.5em;
 }
 
+#pma_navigation #recentTable,
+#pma_navigation #FavoriteTable {
+    width: 200px;
+}
+
 #pma_navigation #pmalogo,
 #pma_navigation #serverChoice,
-#pma_navigation #leftframelinks,
+#pma_navigation #navipanellinks,
 #pma_navigation #recentTableList,
+#pma_navigation #FavoriteTableList,
 #pma_navigation #databaseList,
 #pma_navigation div.pageselector.dbselector {
     text-align:         center;
@@ -88,12 +95,14 @@ if (! defined('PMA_MINIMUM_COMMON') && ! defined('TESTSUITE')) {
 }
 
 #pma_navigation #recentTableList select,
+#pma_navigation #FavoriteTableList select,
 #pma_navigation #serverChoice select
  {
     width: 80%;
 }
 
-#pma_navigation #recentTableList {
+#pma_navigation #recentTableList,
+#pma_navigation #FavoriteTableList {
     margin-bottom: 0;
     padding-bottom: 0;
 }
@@ -118,6 +127,10 @@ if (! defined('PMA_MINIMUM_COMMON') && ! defined('TESTSUITE')) {
     position: absolute;
     height: 100%;
 }
+#pma_navigation_tree_content a.hover_show_full {
+    position: relative;
+    z-index: 9999;
+}
 #pma_navigation_tree a {
     color: <?php echo $GLOBALS['cfg']['NaviColor']; ?>;
 }
@@ -132,6 +145,21 @@ if (! defined('PMA_MINIMUM_COMMON') && ! defined('TESTSUITE')) {
     color: <?php echo $GLOBALS['cfg']['NaviPointerColor']; ?>;
     background-color: <?php echo $GLOBALS['cfg']['NaviPointerBackground']; ?>;
 }
+#pma_navigation_tree li .dbItemControls {
+    padding-left: 4px;
+}
+#pma_navigation_tree li .navItemControls {
+    display: none;
+    padding-left: 4px;
+}
+#pma_navigation_tree li.activePointer .navItemControls {
+    display: inline;
+    opacity: 0.5;
+}
+#pma_navigation_tree li.activePointer .navItemControls:hover {
+    display: inline;
+    opacity: 1.0;
+}
 #pma_navigation_tree ul {
     clear: both;
     padding: 0;
@@ -143,12 +171,16 @@ if (! defined('PMA_MINIMUM_COMMON') && ! defined('TESTSUITE')) {
 }
 #pma_navigation_tree li {
     white-space: nowrap;
+    padding-bottom: 4px;
     clear: both;
     min-height: 16px;
 }
 #pma_navigation_tree img {
     margin: 0;
 }
+#pma_navigation_tree i {
+    display: block;
+}
 #pma_navigation_tree div.block {
     position: relative;
     width:1.5em;
@@ -160,7 +192,7 @@ if (! defined('PMA_MINIMUM_COMMON') && ! defined('TESTSUITE')) {
 #pma_navigation_tree div.block i,
 #pma_navigation_tree div.block b {
     width: 1.5em;
-    height: 1.5em;
+    height: 1.7em;
     min-width: 16px;
     min-height: 8px;
     position: absolute;
@@ -171,11 +203,14 @@ if (! defined('PMA_MINIMUM_COMMON') && ! defined('TESTSUITE')) {
 #pma_navigation_tree div.block i {
     border-<?php echo $left; ?>: 1px solid #666;
     border-bottom: 1px solid #666;
+    position: relative;
+    z-index: 0;
 }
 #pma_navigation_tree div.block i.first { /* Removes top segment */
     border-<?php echo $left; ?>: 0;
 }
-#pma_navigation_tree div.block b { /* Bottom segment for the tree element connections */
+/* Bottom segment for the tree element connections */
+#pma_navigation_tree div.block b {
     display: block;
     height: 0.75em;
     bottom: 0;
@@ -220,7 +255,7 @@ li.fast_filter {
     border-<?php echo $left; ?>: 1px solid #666;
 }
 li.fast_filter input {
-    padding-<?php echo $right; ?>: .4em;
+    padding-<?php echo $right; ?>: 1.7em;
     width: 100%;
 }
 li.fast_filter span {
@@ -231,6 +266,21 @@ li.fast_filter span {
     font-weight: bold;
     color: #800;
 }
+/* IE10+ has its own reset X */
+html.ie li.fast_filter span {
+    display: none;
+}
+html.ie.ie9 li.fast_filter span,
+html.ie.ie8 li.fast_filter span {
+    display: auto;
+}
+html.ie li.fast_filter input {
+    padding-<?php echo $right; ?>: .2em;
+}
+html.ie.ie9 li.fast_filter input,
+html.ie.ie8 li.fast_filter input {
+    padding-<?php echo $right; ?>: 1.7em;
+}
 li.fast_filter.db_fast_filter {
     border: 0;
 }
@@ -263,3 +313,62 @@ li.fast_filter.db_fast_filter {
     filter: dropshadow(color=#fff, offx=0, offy=1);
     border: 1px solid #888;
 }
+
+/* Quick warp links */
+#pma_quick_warp {
+    margin-top: 5px;
+    margin-<?php echo $left; ?>: 2px;
+    position: relative;
+}
+#pma_quick_warp .drop_list {
+    float: <?php echo $left; ?>;
+    margin-<?php echo $left; ?>: 3px;
+    padding: 2px 0;
+}
+#pma_quick_warp .drop_button{
+    padding: 0 .3em;
+    border: 1px solid #ddd;
+    background: #f2f2f2;
+    cursor: pointer;
+}
+#pma_quick_warp .drop_list:hover .drop_button {
+    background: #fff;
+}
+#pma_quick_warp .drop_list ul {
+    position: absolute;
+    margin: 0;
+    padding: 0;
+    overflow: hidden;
+    overflow-y: auto;
+    list-style: none;
+    background: #fff;
+    border: 1px solid #ddd;
+    border-top-<?php echo $right; ?>-radius: 0;
+    border-bottom-<?php echo $right; ?>-radius: 0;
+    top: 100%;
+    <?php echo $left; ?>: 3px;
+    <?php echo $right; ?>: 0;
+    display: none;
+    z-index: 802;
+}
+#pma_quick_warp .drop_list:hover ul {
+    display: block;
+}
+#pma_quick_warp .drop_list li {
+    white-space: nowrap;
+}
+#pma_quick_warp .drop_list li img {
+    vertical-align: sub;
+}
+#pma_quick_warp .drop_list li:hover {
+    background: #f2f2f2;
+}
+#pma_quick_warp .drop_list a {
+    display: block;
+    padding: .1em .3em;
+}
+#pma_quick_warp .drop_list a.favorite_table_anchor {
+    clear: left;
+    float: left;
+    padding: .1em .3em 0;
+}
